@@ -537,10 +537,10 @@ char **argv;
   fdout = ostream->sm.sm_int0;
   if (fork() == 0)
     { /* the child --- replace standard in and out with descriptors given */
-      close(0);
-      dup(fdin);
-      close(1);
-      dup(fdout);
+      dup2(fdin,0);
+      dup2(fdout,1);
+      close(fdin);
+      close(fdout);
       fprintf(stderr, "\n***** Spawning process %s ", pname);
       if (execvp(pname, argv) == -1)
 	{
